function results=control_core(models,inputs)
%CONTROL_CORE - compute the core inversion
%
%   Usage:
%      results=control_core(models,inputs);
%

%recover models
m_dh=models.dh;

%recover parameters common to all solutions
debug=m_dh.parameters.debug;
dim=m_dh.parameters.dim;
ishutter=m_dh.parameters.ishutter;
ismacayealpattyn=m_dh.parameters.ismacayealpattyn;
isstokes=m_dh.parameters.isstokes;

%initialize control parameters, gradients and observations
u_g_obs=m_dh.parameters.u_g_obs;
grad_g=zeros(m_dh.parameters.numberofnodes,1);
param_g=models.dh.parameters.param_g;

%set optimization options.
options=ControlOptions(m_dh.parameters);

for n=1:m_dh.parameters.nsteps,

	%set options
	options=optimset(options,'MaxFunEvals',m_dh.parameters.maxiter(n));

	disp(sprintf('\n%s%s%s%s\n',['   control method step ' num2str(n) '/' num2str(m_dh.parameters.nsteps)]));

	%update inputs with new fit
	inputs=add(inputs,'fit',m_dh.parameters.fit(n),'double');
	inputs=add(inputs,m_dh.parameters.control_type,param_g,'doublevec',1,m_dh.parameters.numberofnodes);

	%Update inputs in datasets
	[m_dh.elements,m_dh.nodes,m_dh.loads,m_dh.materials]=UpdateFromInputs(m_dh.elements,m_dh.nodes,m_dh.loads,m_dh.materials,inputs);

	displaystring(debug,'\n%s',['      computing gradJ...']);
	[u_g c(n).grad_g]=GradJCompute(m_dh,inputs,u_g_obs,'diagnostic','horiz');
	inputs=add(inputs,'velocity',u_g,'doublevec',2,m_dh.parameters.numberofnodes);

	displaystring(debug,'\n%s',['      normalizing directions...']);
	if n>=2,
		c(n).grad_g=Orth(c(n).grad_g,c(n-1).grad_g);
	else
		c(n).grad_g=Orth(c(n).grad_g,{});
	end

	%visualize direction.
	if m_dh.parameters.plot
		plot_direction;
	end

	displaystring(debug,'\n%s',['      optimizing along gradient direction...']);
	[search_scalar c(n).J]=ControlOptimization('objectivefunctionC',0,1,options,m_dh,inputs,param_g,u_g_obs,c(n).grad_g,n,'diagnostic','horiz');

	displaystring(debug,'\n%s',['      updating parameter using optimized search scalar...']);
	param_g=param_g+search_scalar*m_dh.parameters.optscal(n)*c(n).grad_g;

	displaystring(debug,'\n%s',['      constraining the new distribution...']);
	param_g=ControlConstrain(param_g,m_dh.parameters);

	%visualize direction.
	if m_dh.parameters.plot,
		plot_newdistribution;
	end

	disp(['      value of misfit J after optimization #' num2str(n) ':' num2str(c(n).J)]);
end

%generate output
displaystring(debug,'\n%s',['      preparing final velocity solution...']);
results=controlfinalsol(c,m_dh,param_g,inputs,'diagnostic','horiz');
